/*
 * File:		USB_device.c
 * Purpose:		Main process
 *
 */
#include "common.h"     /* Common definitions */
#include "USB_CDC.h"    /* USB CDC support */
#include "USB_Reg.h"    /* USB regulator */

/*****************************************************************
* uint32_t calcCrc32(uint32_t* data, uint32_t len, uint32_t startValue)
*
* Calculates CRC32 from start and len uint32_t's forward.
* If the CRC32 record is included last the return value shall
* be zero if match (OK). Uses the K60 CRC module features.
*
* In:  None
* Out: calculated CRC32
*
*****************************************************************/
uint32_t calcCrc32(uint32_t* data, uint32_t len, uint32_t startValue)
{
    uint32_t result=0;
    uint32_t *memptr;

    memptr = data;

    SIM_SCGC6 |= SIM_SCGC6_CRC_MASK;

    CRC_CTRL = CRC_CTRL_WAS_MASK | CRC_CTRL_TCRC_MASK; /* 32bit CRC      */
    CRC_CRC = startValue;                              /* Seed           */
    CRC_CTRL = CRC_CTRL_TCRC_MASK;                     /* 32bit CRC      */
    CRC_GPOLY = 0x04C11DB7;                            /* Poly to match
                                                          IAR 32-bit default */

    for (uint32_t i=0; i < len; i++)
    {
        CRC_CRC = *memptr++;
    }

    result = CRC_CRC;

    return result;
}

/* Extern Variables */
extern UINT8 gu8USB_Flags;
extern UINT8 gu8EP3_OUT_ODD_Buffer[];
extern tBDT tBDTtable[];

int ok = -1;

extern uint32_t __checksum;

uint32_t valCrc32;

/********************************************************************/
void main (void)
{
    // ROM content
    uint32_t start_address = 0x410;
    uint32_t end_address   = 0x2ffb;

    uint32_t len = (end_address + 1 - start_address) / 4;

    valCrc32 = calcCrc32((uint32_t*)start_address, len, 0x0);

    if (valCrc32 == __checksum)
    {
      ok = 1; // TBD
    }
    else
    {
      ok = 0; // TBD
    }

    // ROM content, including checksum

    len++;

    valCrc32 = calcCrc32((uint32_t*)start_address, len, 0x0);

    if (valCrc32 == 0)
    {
      ok = 1; // TBD
    }
    else
    {
      ok = 0; // TBD
    }

    printf("\n********** USB Device Module Testing **********\n");
    USB_REG_SET_ENABLE;
    USB_REG_CLEAR_STDBY;

    // USB CDC Initialization
    CDC_Init();


    while(1)
    {
        // USB CDC service routine
        CDC_Engine();

        // If data transfer arrives
        if(FLAG_CHK(EP_OUT,gu8USB_Flags))
        {
            (void)USB_EP_OUT_SizeCheck(EP_OUT);
            usbEP_Reset(EP_OUT);
            usbSIE_CONTROL(EP_OUT);
            FLAG_CLR(EP_OUT,gu8USB_Flags);

            // Send it back to the PC
            EP_IN_Transfer(EP2,CDC_OUTPointer,1);
        }

    }
}
